/*
This sketch was written by Will Anderson in 2013. It draws on the Ardumote example sketch by Samrat Amin (http://www.samratamin.com). Several elements have been edited or influenced by Reuben James.
Checkout the instructable (by user willpow1993) that goes with this code. Happy coding!
*/
#include <SPI.h>         // for Arduino later than ver 0018
//#include <EthernetUdp.h>   // UDP library from bjoern@cs.stanford.edu
#include <Ethernet.h>
#include <RCSwitch.h>    // This is the library that will allow us to control the mains switches
#include <Wire.h>    // The wire library allows the arduino to communicate with the clock module using A4 and A5
#include <RTClib.h>  // This is the adafruit version of JeeLab's RTClib library that we will be using to work with the clock module

RCSwitch Switch1 = RCSwitch();  // Think of this as naming the remote as "Switch1"
RTC_DS1307 rtc; // This defines exactly what clock module we're using

// The next few sections are all part of the Ardumote requirements and will allow the iPhone to talk to the ethernet shield and vice-versa
////////////////  NETWORK INFO  ////////////////

byte mac[] = { 0xZZ, 0xZZ, 0xZZ, 0xZZ, 0xZZ, 0xZZ };  //Set your Ethernet Shield's MAC address here - make sure you replace the ZZs with your shield's values!
byte ip[] = {192, 168, 0, 0 };    // Set your shield's desired IP address here - check your network for configuration details
//byte gateway[] = { 192,168,1,1 };   //if you need to set a gateway IP
//byte subnet[] = { 255,255,255,0 };    // Change this to your subnet address
  
unsigned int localPort = 7777;      // local port to listen on (set this the same as Port # on Ardumote Params Screen)

IPAddress iPhoneIP(192, 168, 0, 1);  //Set the iPhone/iPod/iPad's IP address to send messages back to Ardumote...
unsigned int iPhonePort = 7777;      //Set the Port # of the message table you configured in Ardumote (default is 7777)...

///////////////// UDP Variables  ////////////////// 

// the next two variables are set when a packet is received
//byte remoteIp[4];          // holds received packet's originating IP
//unsigned int  remotePort;   // holds received packet's originating port

// buffers for receiving and sending data
char packBuff[UDP_TX_PACKET_MAX_SIZE]; //buffer to hold incoming packet,

/////////////////////////////////////////////////


EthernetUDP Udp;







unsigned long showerStart = 0;            // This is the long (32 bit non negative) storage for the time at which the shower starts (it's stored in seconds from midnight, January 1st 1970!)
int showerActive = 0;                     // This starts the if statements that control the shower runtime. This is simply here to save on processing power (the code bypasses the shower if statments if it's 0)
unsigned long bedStart = 0;               // This is the long (32 bit non negative) storage for the time at which the bed starts (it's stored in seconds from midnight, January 1st 1970!)
int bedActive = 0;                        // This starts the if statements that control the bed runtime. This is simply here to save on processing power (the code bypasses the bed if statments if it's 0)
boolean multiOff = 0;
unsigned long teethStart = 0;             // This is the long (32 bit non negative) storage for the time at which the bed starts (it's stored in seconds from midnight, January 1st 1970!)
int teethActive = 0;                      // This starts the if statements that control the teeth runtime. This is simply here to save on processing power (the code bypasses the teeth if statments if it's 0)







void setup() {
  
  Ethernet.begin(mac,ip);                  // If you don't need to set your default gateway or subnet manually, use this
// Ethernet.begin(mac,ip,gateway,subnet);  // Use this line instead if you've manually set all the parameters
  
  
  Udp.begin(localPort);                   // Setup UDP socket on port defined earlier

  Serial.begin(57600);                    // Start Serial Communications with PC
    
  Switch1.enableTransmit(10);             // This defines which pin the remote (or transmitter) is connected to. IN this case it's pin 10 but change it as necessary
  
  //Now for all the stuff to do with the clock module:
  #ifdef AVR
  Wire.begin();
#else
  Wire1.begin(); // Shield I2C pins connect to alt I2C bus on Arduino Due
#endif
  rtc.begin();

  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(__DATE__, __TIME__));
  }
  
  // End of clock module gubbins
  


}

void loop() 
{

  DateTime timeNow = rtc.now();  // This stores the current time (according to the RTC module). This will allow us to compare the current time with times we might set later
  
  int pwmVal;    // Integer that will hold our PWM values for later use
  
  
  
  
  
  
  
  
      // If any item is turned on or off then cancel all multi commands (this will stop things being turned offunexpectedly etc.)
   
     if (multiOff == 1){
       showerActive = 0;
       bedActive = 0;
       teethActive = 0;
       multiOff = 0;
     }
     
     // If more than one multi is active at any one time then only keep the latest to be activated, activated - simples!
     
     if (showerActive+bedActive+teethActive > 1){                // If more than one of the multi's are active then the sum of their active status' will be greater than one.
       if (showerStart > bedStart && showerStart > teethStart){                  // At which point we need to see which one was activated latest (which one was activated the greatest number of seconds since 01/01/1970)
         bedActive = 0;
         teethActive = 0;
         showerActive = 1;
       }
       
       else if(bedStart > showerStart && bedStart > teethStart){
         showerActive = 0;
         teethActive = 0;
         bedActive = 1;
       }
       
       else if(teethStart > showerStart && teethStart > bedStart){
         showerActive = 0;
         bedActive = 0;
         teethActive = 1;
       }
       
       
     }
         
    
    
    // Some big old if statements that are used for multi commands from the iPhone (see further down)
    
     if (showerActive == 1){
          if (showerStart+60 == timeNow.unixtime()){    // If the current time is one minute after the shower routine was activated
            
            loungeMain(false, false);                                                  //After 1 minute (60s) turn off the lounge lights and lamps
            loungeLamp(false, false);          
        }
        
      if (showerStart+120 == timeNow.unixtime()){                                                                  
            upstairs(false, false);                                                 //After 2 minutes (120s) turn off the upstairs lights
            bedside(false, false);
          }
          
        
        
     if (showerStart+1200 == timeNow.unixtime()){
            bathroom(false, false);                                                 //After a total of 20 minutes (1200s) turn off the bathroom lights
            showerActive = 0;           
          }
          
        }


// If statements for bed active runtime

     if (bedActive == 1){
          if (bedStart+60 == timeNow.unixtime()){    // If the current time is one minute after the bed routine was activated
            
            loungeLamp(false, false);                                                  //After 1 minute (60s) turn off the lounge lights and lamps
            loungeMain(false, false);
            bedActive = 0;          
        }
     }
        
       
       if (teethActive == 1){
          if (teethStart+60 == timeNow.unixtime()){    // If the current time is one minute after the teeth routine was activated
            
            loungeLamp(false, false);                                                  //After 1 minute (60s) turn off the lounge lights and lamps and the bedside lamp
            loungeMain(false, false);
            bedside(false, false);        
        }
        
          if (teethStart+180 == timeNow.unixtime()){    // If the current time is three minutes after the teeth routine was activated
          
              bedside(true, false);                    // Turn on the bedside lamp
          }
          
          if (teethStart+360 == timeNow.unixtime()){    // If the current time is six minutes after the teeth reutine was activated
          
            bathroom(false, false);                    // Turn the bathroom lights off
           
            teethActive = 0;                        // Turn of teethActive
          }
       }
       
       
       
       
       
       
       
       
  
  
  // if there's data available, read a packet
  int packetSize = Udp.parsePacket(); // note that this includes the UDP header
  if(packetSize)
  {
    packetSize = packetSize - 8;      // subtract the 8 byte header
    Serial.print("Packet size: ");
    Serial.println(packetSize);

    // read the packet into packetBuffer and get the senders IP address and port number
    Udp.read(packBuff,UDP_TX_PACKET_MAX_SIZE);
    Serial.println("Message: ");
    Serial.println(packBuff);
    
    
    

          
          
        
        
       
        
        
  

 /* PWM - If we move a slider on Ardumote, it sends in a 3 digit value attached to the message of the slider.
 
     For example, if your message is set to be "PWM" and your slider is halfway set (slider value is 127),
     then your actual sent message will be received as "PWM127".  Therefore, to set the Pin's PWM value, you simply
     extract the last 3 digits of your message and use that as your PWM value (see below):
     
  */
    
    // Assuming our packBuff's contents at index values 3-5 are our PWM values, you can convert them to an int using this:
  
     pwmVal = (packBuff[3] - '0')*100 + (packBuff[4] - '0')*10 + (packBuff[5] - '0');    //Get PWMXXX message, and use XXX to set an int between 0 and 255.
 
 //////////////////////// Pin 6 (LED_Pin) /////////////////////////////////////       
        
     if (packBuff[0] = 'P' && packBuff[1]=='W' && packBuff[2]=='M')  // Wait for "PWMXXX" and use XXX as value for PWM 
    {

      
      //analogWrite(LED_Pin,pwmVal);    //Set LED_Pin to PWM Value
      
      //Serial.println("PWM on Pin 6");    //Write notification  
    

    }
    
    
    
    
    
    /////////////// The following section is for turning on and off individual devices /////////////// 


    // Name of electrical item beeing tunred on or off goes here (e.g. Kitchen Light)
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '1') // If we get the message "P01"something, then get the arduino to deal with switch 1 of group 3
    {
      if (packBuff[3] == 'H'){
        kitchen(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on) and activate multiOff
      } else {
        kitchen(false, true);                            // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off) and activate multiOff
      }
      
    }
    
    // Outside Light
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '2')     // If we get the message "P02"something, then get the arduino to deal with switch 2 of group 3
    {
      if (packBuff[3] == 'H') {
        outside(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        outside(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Lounge Lights
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '3')
    {
      if (packBuff[3] == 'H') {
        loungeMain(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        loungeMain(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Desk lamp
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '4')
   {
      if (packBuff[3] == 'H') {
        desk(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        desk(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Lounge Lamps
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '5')
    {
      if (packBuff[3] == 'H') {
        loungeLamp(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        loungeLamp(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Upstairs Lights
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '6')
    {
      if (packBuff[3] == 'H') {
        bathroom(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        bathroom(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Bathroom Lights
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '7')
    {
      if (packBuff[3] == 'H') {
        upstairs(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        upstairs(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Bedside Lamp
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '8')
    {
      if (packBuff[3] == 'H') {
        bedside(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        bedside(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Xbox + TV
    else if (packBuff[0] = 'P' && packBuff[1] == '0' && packBuff[2] == '9')
    {
      if (packBuff[3] == 'H') {
        xboxTV(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        xboxTV(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Sound System
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '0')
    {
      if (packBuff[3] == 'H') {
        sound(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        sound(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Kettle
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '1')
    {
      if (packBuff[3] == 'H') {
        kettle(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        kettle(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    // Boiler
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '2')
    {
      if (packBuff[3] == 'H') {
        boiler(true, true);                             // Turn the switch On if the On message has been recieved (ardumote sends xxxH if on)
      } else {
        boiler(false, true) ;                           // Turn the switch Off if the Off message has been recieved (ardumote sends xxxL if off)
      }
    }
    
    
    
    
    /////////////// The following section is for commands that will turn on or off multipul devices ///////////////
    
    
    // Turn all lights on or off
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '3')
    {
      if (packBuff[3] == 'H') {
        allLights(true);     
      } 
      
      else {
        allLights(false);      
      }
    }
    
    
    /* "I'm going for a shower" - This will turn on the boiler, turn off the lounge lights and lamps after 1 minute, it will turn on the upstairs lights for 2 minutes before turning them off again, it will
   turn on the bathroom lights for 20 mins before turning them off again. All other lights and appliances will be turned off imediately.
   */
   
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '4')
    {
        shower(timeNow);
    }
    
    
     /* "I'm going to bed" - This will turn on the bed side lamp, turn off the lounge lights and lamps after 1 minute, it will turn of all other lights and appliances imediately (I'm thinking I mihgt put in an
     extra perameter that means the boiler won't be turned off in the winter but that's for another day)
   */
    
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '5')
    {
        bed(timeNow);                                                     
               
    }
    
    
    
    
         /* "I'm going to sleep" - This will turn off everything!
   */
    
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '6')
    {
      sleeping(timeNow);
               
    }

    
    
         /* Entertainment - This will turn on the xbox and TV and the sound system. If it is past 6pm then it will turn on the lounge lamps. Everything else will be turned off other than the boiler.
   */
    
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '8')
    {
       entertainment(timeNow);
    }
    
    
    
    
         /* "I'm going to clean my teeth" - This will turn on the bathroom light for six minutes, it will turn on the bedside lamp for 1 minute, turn off the lounge lights and lamps after 1 minute, 
         it will then turn on the bedside light after a total of three minutes from the time the run time was activated.
   */
    
    else if (packBuff[0] = 'P' && packBuff[1] == '1' && packBuff[2] == '9')
    {
        teeth(timeNow);
    }
    
    
  }
  
  
  delay(20);
}




/* This is the method "setOnOff" which deals with all the nitty gritty of turning on or off specific switches. It also allows for serial printing and feedback to the iPhone of what has been turned on or off.
It was created by Reuben James */
void setOnOff(int group, int plug, boolean on) {
  
  char* s = "Switch ";
  s += char(group) + (',', ' ') + char(plug) + (' ', 'i', 's', ' ', 'O'); // ON and OFF both start with O
  if (on) {
    s += ('N');
    Switch1.switchOn(group, plug);
  } else {
    s += ('F', 'F');
    Switch1.switchOff(group, plug);
  }
  
  Serial.println(s);
  
  // UDP returns a message to the sender (iPhone) saying what has just happened. DOESN'T WORK YET!!!
  /*Udp.beginPacket(iPhoneIP, iPhonePort);
  Udp.write(s);    // Send Message back to iPhone
  Udp.endPacket();*/
}

void kitchen (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(3, 1, on);
}


void outside (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(3, 2, on);
  
}


void loungeMain (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(3, 3, on);
}

void desk (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(3, 4, on);
}

void loungeLamp (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(2, 1, on);
} 
  
  
void upstairs (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(2, 2, on);    
}

void bathroom (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(2, 3, on);
}
  
 void bedside (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(2, 4, on);
}


void xboxTV (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(1, 1, on);
} 
  
  
void sound (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(1, 2, on);    
}

void kettle (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(1, 3, on);
}
  
 void boiler (boolean on, boolean multioff){
  multiOff = multioff;
  setOnOff(1, 4, on);
}

void allLights(boolean on){
    kitchen(on, true);
    outside(on, true);
    loungeMain(on, true);
    desk(on, true);
    loungeLamp(on, true);
    upstairs(on, true);
    bathroom(on, true);
    bedside(on, true);
}

void shower(DateTime timeNow){
  
        upstairs(true, false);                                                   //Turn on the upstirs lights, the bathroom lights, the bedside lamp, and the boiler
        bathroom(true, false);
        bedside(true, false);
        boiler(true, false); 
    
    
        kitchen(false, false);                                                  //Turn most things off
        outside(false, false);
        
        desk(false, false);
        
        
         
        xboxTV(false, false);
        sound(false, false);
        kettle(false, false);
    
        
        showerStart = timeNow.unixtime();                                          // Store the current time
        showerActive = 1;                                                      // Set the showerActive boolean to true causing the if statement that controls the shower runtime to come into effect
        
}

void bed(DateTime timeNow){
        
        bedside(true, false);
        kitchen(false, false);
        outside(false, false);
        
        desk(false, false);
        
        upstairs(false, false);
        bathroom(false, false); 
        
        xboxTV(false, false);
        sound(false, false);
        kettle(false, false);
        boiler(false, false); 
        
        
        
        
        
        
        
        
        bedStart = timeNow.unixtime();                                          // Store the current time
        bedActive = 1;        // Set the bedActive boolean to true causing the if statement that controls the bed runtime to come into effect
}

void sleeping(DateTime timeNow){
        kitchen(false, true);
        outside(false, true);
        loungeMain(false, true);
        desk(false, true);
        loungeLamp(false, true);
        upstairs(false, true);
        bathroom(false, true);
        bedside(false, true);
        xboxTV(false, true);
        sound(false, true);
        kettle(false, true);
        boiler(false, true);
                                                                        /* As there is no timing element to this multi there is no need to have a separate set of timing if statments and other multis
                                                                              cause problems with this one. As such, setting multi off will sufice for this purpose. */     
        

}   

void entertainment(DateTime timeNow){
        xboxTV(true, true);
        sound(true, true);
        
        if (timeNow.hour() >= 18 || timeNow.hour() <= 6){
          loungeLamp(true, true);
        }
        else {
          loungeLamp(false, true);
        }
  
  
        kitchen(false, true);                                                  //Turn most things off
        outside(false, true);
        loungeMain(false, true);
        desk(false, true);
        
        upstairs(false, true);
        bathroom(false, true); 
        bedside(false, true);
        
        
        kettle(false, true);
         
        
        
              
        
        multiOff = 1;         /* As there is no timing element to this multi there is no need to have a separate set of timing if statments and other multis
                                                                              cause problems with this one. As such, setting multi off will sufice for this purpose. */        
               
}

void teeth(DateTime timeNow){
        bathroom(true, false);
        bedside(true, false);
  
  
  
        
  
        kitchen(false, false);                                                  //Turn most things off
        outside(false, false);
        
        desk(false, false);
        
        upstairs(false, false);
         
        
        xboxTV(false, false);
        sound(false, false);
        kettle(false, false);
        boiler(false, false); 
  
  
  
        teethStart = timeNow.unixtime();                                          // Store the current time
        teethActive = 1;                                                      // Set the bedActive boolean to true causing the if statement that controls the bed runtime to come into effect
                
}




/*
        kitchen(true, true);
        outside(true, true);
        loungeMain(true, true);
        desk(true, true);
        loungeLamp(true, true);
        upstairs(true, true);
        bathroom(true, true);
        bedside(true, true);
        xboxTV(true, true);
        sound(true, true);
        kettle(true, true);
        boiler(true, true);
    
        kitchen(false, true);
        outside(false, true);
        loungeMain(false, true);
        desk(false, true);
        loungeLamp(false, true);
        upstairs(false, true);
        bathroom(false, true);
        bedside(false, true);
        xboxTV(false, true);
        sound(false, true);
        kettle(false, true);
        boiler(false, true);
    */

